@@ -1,7 +1,8 @@
-
+from dataclasses import dataclass, field
 from io import StringIO, BytesIO
 from os import listdir, mkdir
 from os.path import join
+from typing import Any, Dict, List
 
 from boto3 import Session
 from git import Repo, exc
@@ -18,166 +19,200 @@ from delphi_utils.nancodes import Nans
 
 CSV_DTYPES = {
     "geo_id": str, "val": float, "se": float, "sample_size": float,
-    "missing_val": int, "missing_se": int, "missing_sample_size": int
-    }
-
-CSVS_BEFORE = {
-    # All rows unchanged
-    "csv0": pd.DataFrame({
-        "geo_id": ["1", "2", "3"],
-        "val": [1.000000001, 2.00000002, 3.00000003],
-        "se": [0.1, 0.2, 0.3],
-        "sample_size": [10.0, 20.0, 30.0],
-        "missing_val": [Nans.NOT_MISSING] * 3,
-        "missing_se": [Nans.NOT_MISSING] * 3,
-        "missing_sample_size": [Nans.NOT_MISSING] * 3,
-        }),
-
-    # One row deleted and one row added
-    "csv1": pd.DataFrame({
-        "geo_id": ["1", "2", "3"],
-        "val": [1.0, 2.0, 3.0],
-        "se": [np.nan, 0.20000002, 0.30000003],
-        "sample_size": [10.0, 20.0, 30.0],
-        "missing_val": [Nans.NOT_MISSING] * 3,
-        "missing_se": [Nans.NOT_MISSING] * 3,
-        "missing_sample_size": [Nans.NOT_MISSING] * 3,
-        }),
-
-    # File deleted
-    "csv2": pd.DataFrame({
-        "geo_id": ["1"],
-        "val": [1.0],
-        "se": [0.1],
-        "sample_size": [10.0],
-        "missing_val": [Nans.NOT_MISSING],
-        "missing_se": [Nans.NOT_MISSING],
-        "missing_sample_size": [Nans.NOT_MISSING],
-        }),
-
-    # All rows common, but missing columns added
-    "csv4": pd.DataFrame({
-        "geo_id": ["1"],
-        "val": [1.0],
-        "se": [0.1],
-        "sample_size": [10.0]
-        }),
-
-    # All rows common, but missing columns removed
-    "csv5": pd.DataFrame({
-        "geo_id": ["1"],
-        "val": [1.0],
-        "se": [0.1],
-        "sample_size": [10.0],
-        "missing_val": [Nans.NOT_MISSING],
-        "missing_se": [Nans.NOT_MISSING],
-        "missing_sample_size": [Nans.NOT_MISSING],
-        }),
-
-    # All rows common, but no missing columns
-    "csv6": pd.DataFrame({
-        "geo_id": ["1"],
-        "val": [1.0],
-        "se": [0.1],
-        "sample_size": [10.0]
-        }),
-
-    # Row deleted and row added, but no missing columns (will not be uploaded)
-    "csv7": pd.DataFrame({
-        "geo_id": ["1", "2"],
-        "val": [1.0, 2.0],
-        "se": [0.1, 0.2],
-        "sample_size": [10.0, 20.0]
-        }),
-
-    # Row deleted and row added, but no missing columns
-    "csv8": pd.DataFrame({
-        "geo_id": ["1", "2"],
-        "val": [1.0, 2.0],
-        "se": [0.1, 0.2],
-        "sample_size": [10.0, 20.0]
-        }),
+    "missing_val": "Int64", "missing_se": "Int64", "missing_sample_size": "Int64"
 }
 
-CSVS_AFTER = {
-    # All rows unchanged
-    "csv0": pd.DataFrame({
-        "geo_id": ["1", "2", "3"],
-        "val": [1.0, 2.0, 3.0],
-        "se": [0.10000001, 0.20000002, 0.30000003],
-        "sample_size": [10.0, 20.0, 30.0],
-        "missing_val": [Nans.NOT_MISSING] * 3,
-        "missing_se": [Nans.NOT_MISSING] * 3,
-        "missing_sample_size": [Nans.NOT_MISSING] * 3,
-        }),
+class Example:
+    def __init__(self, before, after, diff):
+        def fix_df(df):
+            if isinstance(df, pd.DataFrame):
+                return Example._set_df_datatypes(df, CSV_DTYPES)
+            return df
+        self.before = fix_df(before)
+        self.after = fix_df(after)
+        self.diff = fix_df(diff)
+    @staticmethod
+    def _set_df_datatypes(df: pd.DataFrame, dtypes: Dict[str, Any]) -> pd.DataFrame:
+        df = df.copy()
+        for k, v in dtypes.items():
+            if k in df.columns:
+                df[k] = df[k].astype(v)
+        return df
 
-    # One row deleted and one row added
-    "csv1": pd.DataFrame({
-        "geo_id": ["1", "2", "4"],
-        "val": [1.0, 2.1, 4.0],
-        "se": [np.nan, 0.21, np.nan],
-        "sample_size": [10.0, 21.0, 40.0],
-        "missing_val": [Nans.NOT_MISSING] * 3,
-        "missing_se": [Nans.NOT_MISSING] * 3,
-        "missing_sample_size": [Nans.NOT_MISSING] * 3,
-        }),
+@dataclass
+class Expecteds:
+    deleted: List[str]
+    common_diffs: Dict[str, str]
+    new: List[str]
+    raw_exports: List[str] = field(init=False)
+    diffed_exports: List[str] = field(init=False)
+    filtered_exports: List[str] = field(init=False)
 
-    # File added
-    "csv3": pd.DataFrame({
-        "geo_id": ["2"],
-        "val": [2.0000002],
-        "se": [0.2],
-        "sample_size": [20.0],
-        "missing_val": [Nans.NOT_MISSING],
-        "missing_se": [Nans.NOT_MISSING],
-        "missing_sample_size": [Nans.NOT_MISSING],
+    def __post_init__(self):
+        self.raw_exports = list(self.common_diffs.keys()) + self.new
+        self.diffed_exports = self.raw_exports + [diff_name for diff_name in self.common_diffs.values() if diff_name is not None]
+        self.filtered_exports = [f.replace(".diff", "") for f in self.diffed_exports if f.endswith(".diff")] + self.new
+EMPTY = "empty"
+CSVS = {
+    "unchanged": Example( # was: csv0
+        before=pd.DataFrame({
+            "geo_id": ["1", "2", "3"],
+            "val": [1.000000001, 2.00000002, 3.00000003],
+            "se": [0.1, 0.2, 0.3],
+            "sample_size": [10.0, 20.0, 30.0],
+            "missing_val": [Nans.NOT_MISSING] * 3,
+            "missing_se": [Nans.NOT_MISSING] * 3,
+            "missing_sample_size": [Nans.NOT_MISSING] * 3,
         }),
-
-    # All rows common, but missing columns added
-    "csv4": pd.DataFrame({
-        "geo_id": ["1"],
-        "val": [1.0],
-        "se": [0.1],
-        "sample_size": [10.0],
-        "missing_val": [Nans.NOT_MISSING],
-        "missing_se": [Nans.NOT_MISSING],
-        "missing_sample_size": [Nans.NOT_MISSING],
+        after=pd.DataFrame({
+            "geo_id": ["1", "2", "3"],
+            "val": [1.0, 2.0, 3.0], # ignore changes beyond 7 decimal points
+            "se": [0.10000001, 0.20000002, 0.30000003],
+            "sample_size": [10.0, 20.0, 30.0],
+            "missing_val": [Nans.NOT_MISSING] * 3,
+            "missing_se": [Nans.NOT_MISSING] * 3,
+            "missing_sample_size": [Nans.NOT_MISSING] * 3,
         }),
-
-    # All rows common, but missing columns removed
-    "csv5": pd.DataFrame({
-        "geo_id": ["1"],
-        "val": [1.0],
-        "se": [0.1],
-        "sample_size": [10.0]
+        diff=EMPTY # unchanged names are listed in common files but have no diff file
+    ),
+    "mod_2_del_3_add_4": Example( # was: csv1
+        before=pd.DataFrame({
+            "geo_id": ["1", "2", "3"],
+            "val": [1.0, 2.0, 3.0],
+            "se": [np.nan, 0.20000002, 0.30000003],
+            "sample_size": [10.0, 20.0, 30.0],
+            "missing_val": [Nans.NOT_MISSING] * 3,
+            "missing_se": [Nans.CENSORED] + [Nans.NOT_MISSING] * 2,
+            "missing_sample_size": [Nans.NOT_MISSING] * 3,
         }),
-
-    # All rows common, but no missing columns
-    "csv6": pd.DataFrame({
-        "geo_id": ["1"],
-        "val": [1.0],
-        "se": [0.1],
-        "sample_size": [10.0]
+        after=pd.DataFrame({
+            "geo_id": ["1", "2", "4"],
+            "val": [1.0, 2.1, 4.0],
+            "se": [np.nan, 0.21, np.nan],
+            "sample_size": [10.0, 21.0, 40.0],
+            "missing_val": [Nans.NOT_MISSING] * 3,
+            "missing_se": [Nans.CENSORED] + [Nans.NOT_MISSING] * 2,
+            "missing_sample_size": [Nans.NOT_MISSING] * 3,
         }),
-
-    # Row deleted and row added, but no missing columns (will not be uploaded)
-    "csv7": pd.DataFrame({
-        "geo_id": ["1"],
-        "val": [1.0],
-        "se": [0.1],
-        "sample_size": [10.0]
+        diff=pd.DataFrame({
+            "geo_id": ["2", "3", "4"],
+            "val": [2.1, np.nan, 4.0],
+            "se": [0.21, np.nan, np.nan],
+            "sample_size": [21.0, np.nan, 40.0],
+            "missing_val": [Nans.NOT_MISSING, Nans.DELETED, Nans.NOT_MISSING],
+            "missing_se": [Nans.NOT_MISSING, Nans.DELETED, Nans.NOT_MISSING],
+            "missing_sample_size": [Nans.NOT_MISSING, Nans.DELETED, Nans.NOT_MISSING],
+        })
+    ),
+    "delete_file": Example( # was: csv2
+        before=pd.DataFrame({
+            "geo_id": ["1"],
+            "val": [1.0],
+            "se": [0.1],
+            "sample_size": [10.0],
+            "missing_val": [Nans.NOT_MISSING],
+            "missing_se": [Nans.NOT_MISSING],
+            "missing_sample_size": [Nans.NOT_MISSING],
         }),
-
-    # Row deleted and row added, but no missing columns
-    "csv8": pd.DataFrame({
-        "geo_id": ["1", "3"],
-        "val": [1.0, 3.0],
-        "se": [0.1, 0.3],
-        "sample_size": [10.0, 30.0]
+        after=None,
+        diff=None
+    ),
+    "add_file": Example( # was: csv3
+        before=None,
+        after=pd.DataFrame({
+            "geo_id": ["2"],
+            "val": [2.0000002],
+            "se": [0.2],
+            "sample_size": [20.0],
+            "missing_val": [Nans.NOT_MISSING],
+            "missing_se": [Nans.NOT_MISSING],
+            "missing_sample_size": [Nans.NOT_MISSING],
         }),
+        diff=None
+    ),
+    "unchanged_old_new": Example( # was: csv4
+        before=pd.DataFrame({
+            "geo_id": ["1"],
+            "val": [1.0],
+            "se": [0.1],
+            "sample_size": [10.0]
+        }),
+        after=pd.DataFrame({
+            "geo_id": ["1"],
+            "val": [1.0],
+            "se": [0.1],
+            "sample_size": [10.0],
+            "missing_val": [Nans.NOT_MISSING],
+            "missing_se": [Nans.NOT_MISSING],
+            "missing_sample_size": [Nans.NOT_MISSING],
+        }),
+        diff=pd.DataFrame({
+            "geo_id": ["1"],
+            "val": [1.0],
+            "se": [0.1],
+            "sample_size": [10.0],
+            "missing_val": [Nans.NOT_MISSING],
+            "missing_se": [Nans.NOT_MISSING],
+            "missing_sample_size": [Nans.NOT_MISSING],
+        })
+    ),
+    "mod_2_del_3_add_4_old_old": Example( # was: csv5
+        before=pd.DataFrame({
+            "geo_id": ["1", "2", "3"],
+            "val": [1.0, 2.0, 3.0],
+            "se": [np.nan, 0.20000002, 0.30000003],
+            "sample_size": [10.0, 20.0, 30.0]
+        }),
+        after=pd.DataFrame({
+            "geo_id": ["1", "2", "4"],
+            "val": [1.0, 2.1, 4.0],
+            "se": [np.nan, 0.21, np.nan],
+            "sample_size": [10.0, 21.0, 40.0]
+        }),
+        diff=pd.DataFrame({
+            "geo_id": ["2", "3", "4"],
+            "val": [2.1, np.nan, 4.0],
+            "se": [0.21, np.nan, np.nan],
+            "sample_size": [21.0, np.nan, 40.0],
+            "missing_val": [np.nan, Nans.DELETED, np.nan],
+            "missing_se": [np.nan, Nans.DELETED, np.nan],
+            "missing_sample_size": [np.nan, Nans.DELETED, np.nan],
+        })
+    )
 }
+EXPECTEDS = Expecteds(
+    deleted=["delete_file.csv"],
+    common_diffs=dict(
+        (f"{csv_name}.csv", None if dfs.diff is EMPTY else f"{csv_name}.csv.diff")
+        for csv_name, dfs in CSVS.items() if dfs.diff is not None
+    ),
+    new=["add_file.csv"],
+)
+# check for incomplete modifications to tests
+assert set(EXPECTEDS.new) == set(f"{csv_name}.csv" for csv_name, dfs in CSVS.items() if dfs.before is None), \
+    "Bad programmer: added more new files to CSVS.after without updating EXPECTEDS.new"
 
-class TestArchiveDiffer:
+def _assert_frames_equal_ignore_row_order(df1, df2, index_cols: List[str] = None):
+    return assert_frame_equal(df1.set_index(index_cols).sort_index(), df2.set_index(index_cols).sort_index())
+
+class ArchiveDifferTestlike:
+    def set_up(self, tmp_path):
+        cache_dir = join(str(tmp_path), "cache")
+        export_dir = join(str(tmp_path), "export")
+        mkdir(cache_dir)
+        mkdir(export_dir)
+        return cache_dir, export_dir
+    def check_filtered_exports(self, export_dir):
+        assert set(listdir(export_dir)) == set(EXPECTEDS.filtered_exports)
+        for f in EXPECTEDS.filtered_exports:
+            example = CSVS[f.replace(".csv", "")]
+            _assert_frames_equal_ignore_row_order(
+                pd.read_csv(join(export_dir, f), dtype=CSV_DTYPES),
+                example.after if example.diff is None else example.diff,
+                index_cols=["geo_id"]
+            )
+
+class TestArchiveDiffer(ArchiveDifferTestlike):
 
     def test_stubs(self):
         arch_diff = ArchiveDiffer("cache", "export")
@@ -189,20 +224,7 @@ class TestArchiveDiffer:
             arch_diff.archive_exports(None)
 
     def test_diff_and_filter_exports(self, tmp_path):
-        cache_dir = join(str(tmp_path), "cache")
-        export_dir = join(str(tmp_path), "export")
-        mkdir(cache_dir)
-        mkdir(export_dir)
-
-        csv1_diff = pd.DataFrame({
-            "geo_id": ["3", "2", "4"],
-            "val": [np.nan, 2.1, 4.0],
-            "se": [np.nan, 0.21, np.nan],
-            "sample_size": [np.nan, 21.0, 40.0],
-            "missing_val": [Nans.DELETED] + [Nans.NOT_MISSING] * 2,
-            "missing_se": [Nans.DELETED] + [Nans.NOT_MISSING] * 2,
-            "missing_sample_size": [Nans.DELETED] + [Nans.NOT_MISSING] * 2,
-        })
+        cache_dir, export_dir = self.set_up(tmp_path)
 
         arch_diff = ArchiveDiffer(cache_dir, export_dir)
 
@@ -214,37 +236,39 @@ class TestArchiveDiffer:
             arch_diff.diff_exports()
 
         # Simulate cache updated, and signal ran finish
-        for csv_name, df in CSVS_BEFORE.items():
-            df.to_csv(join(cache_dir, f"{csv_name}.csv"), index=False)
-        for csv_name, df in CSVS_AFTER.items():
-            df.to_csv(join(export_dir, f"{csv_name}.csv"), index=False)
+        for csv_name, dfs in CSVS.items():
+            if dfs.before is not None:
+                dfs.before.to_csv(join(cache_dir, f"{csv_name}.csv"), index=False)
+            if dfs.after is not None:
+                dfs.after.to_csv(join(export_dir, f"{csv_name}.csv"), index=False)
         arch_diff._cache_updated = True
 
         deleted_files, common_diffs, new_files = arch_diff.diff_exports()
 
-        # Check return values
-        assert set(deleted_files) == {join(cache_dir, "csv2.csv")}
+        # Check deleted, common, diffed, and new file names match expected
+        assert set(deleted_files) == {join(cache_dir, f) for f in EXPECTEDS.deleted}
         assert set(common_diffs.keys()) == {
-            join(export_dir, f) for f in ["csv0.csv", "csv1.csv", "csv4.csv", "csv5.csv", "csv6.csv", "csv7.csv", "csv8.csv"]}
-        assert set(new_files) == {join(export_dir, "csv3.csv")}
-        assert common_diffs[join(export_dir, "csv0.csv")] is None
-        assert common_diffs[join(export_dir, "csv1.csv")] == join(
-            export_dir, "csv1.csv.diff")
+            join(export_dir, csv_name) for csv_name in EXPECTEDS.common_diffs}
+        assert set(new_files) == {join(export_dir, f) for f in EXPECTEDS.new}
+        # Check that diffed file names are identical
+        assert all(
+            (common_diffs[join(export_dir, csv_name)] ==
+             None if diff_name is None else join(export_dir, diff_name))
+            for csv_name, diff_name in EXPECTEDS.common_diffs.items()
+        )
 
         # Check filesystem for actual files
-        assert set(listdir(export_dir)) == {
-            "csv0.csv",
-            "csv1.csv", "csv1.csv.diff",
-            "csv3.csv",
-            "csv4.csv", "csv4.csv.diff",
-            "csv5.csv", "csv5.csv.diff",
-            "csv6.csv",
-            "csv7.csv", "csv7.csv.diff",
-            "csv8.csv", "csv8.csv.diff"
-        }
-        assert_frame_equal(
-            pd.read_csv(join(export_dir, "csv1.csv.diff"), dtype=CSV_DTYPES),
-            csv1_diff)
+        assert set(listdir(export_dir)) == set(EXPECTEDS.diffed_exports)
+
+        # Check that the diff files look as expected
+        for key, diff_name in EXPECTEDS.common_diffs.items():
+            if diff_name is None: continue
+            _assert_frames_equal_ignore_row_order(
+                pd.read_csv(join(export_dir, diff_name), dtype=CSV_DTYPES),
+                CSVS[key.replace(".csv", "")].diff,
+                index_cols=["geo_id"]
+            )
+
 
         # Test filter_exports
         # ===================
@@ -259,37 +283,24 @@ class TestArchiveDiffer:
         arch_diff.filter_exports(common_diffs)
 
         # Check exports directory just has incremental changes
-        assert set(listdir(export_dir)) == {"csv1.csv", "csv3.csv", "csv4.csv", "csv5.csv", "csv7.csv", "csv8.csv"}
-        assert_frame_equal(
-            pd.read_csv(join(export_dir, "csv1.csv"), dtype=CSV_DTYPES),
-            csv1_diff)
-
+        self.check_filtered_exports(export_dir)
 
 AWS_CREDENTIALS = {
     "aws_access_key_id": "FAKE_TEST_ACCESS_KEY_ID",
     "aws_secret_access_key": "FAKE_TEST_SECRET_ACCESS_KEY",
 }
 
-
-@pytest.fixture(scope="function")
-def s3_client():
-    with mock_s3():
-        yield Session(**AWS_CREDENTIALS).client("s3")
-
-
-class TestS3ArchiveDiffer:
+class TestS3ArchiveDiffer(ArchiveDifferTestlike):
     bucket_name = "test-bucket"
     indicator_prefix = "test"
 
     @mock_s3
-    def test_update_cache(self, tmp_path, s3_client):
-        cache_dir = join(str(tmp_path), "cache")
-        export_dir = join(str(tmp_path), "export")
-        mkdir(cache_dir)
-        mkdir(export_dir)
+    def test_update_cache(self, tmp_path):
+        s3_client = Session(**AWS_CREDENTIALS).client("s3")
+        cache_dir, export_dir = self.set_up(tmp_path)
 
-        csv1 = CSVS_BEFORE["csv1"]
-        csv2 = CSVS_AFTER["csv1"]
+        csv1 = CSVS["mod_2_del_3_add_4"].before
+        csv2 = CSVS["mod_2_del_3_add_4"].after
         csv1_buf = StringIO()
         csv2_buf = StringIO()
         csv1.to_csv(csv1_buf, index=False)
@@ -320,13 +331,11 @@ class TestS3ArchiveDiffer:
         assert set(listdir(cache_dir)) == {"csv1.csv", "csv2.csv"}
 
     @mock_s3
-    def test_archive_exports(self, tmp_path, s3_client):
-        cache_dir = join(str(tmp_path), "cache")
-        export_dir = join(str(tmp_path), "export")
-        mkdir(cache_dir)
-        mkdir(export_dir)
+    def test_archive_exports(self, tmp_path):
+        s3_client = Session(**AWS_CREDENTIALS).client("s3")
+        cache_dir, export_dir = self.set_up(tmp_path)
 
-        csv1 = CSVS_BEFORE["csv1"]
+        csv1 = CSVS["mod_2_del_3_add_4"].before
         csv1.to_csv(join(export_dir, "csv1.csv"), index=False)
 
         s3_client.create_bucket(Bucket=self.bucket_name)
@@ -351,25 +360,23 @@ class TestS3ArchiveDiffer:
         assert_frame_equal(pd.read_csv(body, dtype=CSV_DTYPES), csv1)
 
     @mock_s3
-    def test_run(self, tmp_path, s3_client):
-        cache_dir = join(str(tmp_path), "cache")
-        export_dir = join(str(tmp_path), "export")
-        mkdir(cache_dir)
-        mkdir(export_dir)
+    def test_run(self, tmp_path):
+        s3_client = Session(**AWS_CREDENTIALS).client("s3")
+        cache_dir, export_dir = self.set_up(tmp_path)
 
-        # Set up current buckets to be `CSVS_BEFORE`.
         s3_client.create_bucket(Bucket=self.bucket_name)
-        for csv_name, df in CSVS_BEFORE.items():
-            csv_buf = StringIO()
-            df.to_csv(csv_buf, index=False)
-            s3_client.put_object(
-                Bucket=self.bucket_name,
-                Key=f"{self.indicator_prefix}/{csv_name}.csv",
-                Body=BytesIO(csv_buf.getvalue().encode()))
-
-        # Set up the exported files to be `CSVS_AFTER`.
-        for csv_name, df in CSVS_AFTER.items():
-            df.to_csv(join(export_dir, f"{csv_name}.csv"), index=False)
+        for csv_name, dfs in CSVS.items():
+            # Set up current buckets to be the 'before' files.
+            if dfs.before is not None:
+                csv_buf = StringIO()
+                dfs.before.to_csv(csv_buf, index=False)
+                s3_client.put_object(
+                    Bucket=self.bucket_name,
+                    Key=f"{self.indicator_prefix}/{csv_name}.csv",
+                    Body=BytesIO(csv_buf.getvalue().encode()))
+            # Set up the exported files to be the 'after' files.
+            if dfs.after is not None:
+                dfs.after.to_csv(join(export_dir, f"{csv_name}.csv"), index=False)
 
         # Create and run differ.
         arch_diff = S3ArchiveDiffer(
@@ -379,33 +386,19 @@ class TestS3ArchiveDiffer:
         arch_diff.run()
 
         # Check that the buckets now contain the exported files.
-        for csv_name, df in CSVS_AFTER.items():
+        for csv_name, dfs in CSVS.items():
+            if dfs.after is None:
+                continue
             body = s3_client.get_object(Bucket=self.bucket_name, Key=f"{self.indicator_prefix}/{csv_name}.csv")["Body"]
-            assert_frame_equal(pd.read_csv(body, dtype=CSV_DTYPES), df)
+            assert_frame_equal(pd.read_csv(body, dtype=CSV_DTYPES), dfs.after)
 
         # Check exports directory just has incremental changes
-        assert set(listdir(export_dir)) == {"csv1.csv", "csv3.csv", "csv4.csv", "csv5.csv", "csv7.csv", "csv8.csv"}
-        csv1_diff = pd.DataFrame({
-            "geo_id": ["3", "2", "4"],
-            "val": [np.nan, 2.1, 4.0],
-            "se": [np.nan, 0.21, np.nan],
-            "sample_size": [np.nan, 21.0, 40.0],
-            "missing_val": [Nans.DELETED] + [Nans.NOT_MISSING] * 2,
-            "missing_se": [Nans.DELETED] + [Nans.NOT_MISSING] * 2,
-            "missing_sample_size": [Nans.DELETED] + [Nans.NOT_MISSING] * 2,
-            })
-        assert_frame_equal(
-            pd.read_csv(join(export_dir, "csv1.csv"), dtype=CSV_DTYPES),
-            csv1_diff)
+        self.check_filtered_exports(export_dir)
 
-
-class TestGitArchiveDiffer:
+class TestGitArchiveDiffer(ArchiveDifferTestlike):
 
     def test_init_args(self, tmp_path):
-        cache_dir = str(tmp_path / "cache")
-        export_dir = str(tmp_path / "export")
-        mkdir(cache_dir)
-        mkdir(export_dir)
+        cache_dir, export_dir = self.set_up(tmp_path)
 
         with pytest.raises(AssertionError):
             GitArchiveDiffer(cache_dir, export_dir,
@@ -421,10 +414,7 @@ class TestGitArchiveDiffer:
         assert arch_diff.branch == arch_diff.repo.active_branch
 
     def test_update_cache(self, tmp_path):
-        cache_dir = str(tmp_path / "cache")
-        export_dir = str(tmp_path / "export")
-        mkdir(cache_dir)
-        mkdir(export_dir)
+        cache_dir, export_dir = self.set_up(tmp_path)
 
         Repo.init(cache_dir)
 
@@ -443,10 +433,7 @@ class TestGitArchiveDiffer:
         assert arch_diff2._cache_updated
 
     def test_diff_exports(self, tmp_path):
-        cache_dir = str(tmp_path / "cache")
-        export_dir = str(tmp_path / "export")
-        mkdir(cache_dir)
-        mkdir(export_dir)
+        cache_dir, export_dir = self.set_up(tmp_path)
 
         branch_name = "test-branch"
 
@@ -479,7 +466,7 @@ class TestGitArchiveDiffer:
             "missing_val": [Nans.NOT_MISSING] * 3,
             "missing_se": [Nans.NOT_MISSING] * 3,
             "missing_sample_size": [Nans.NOT_MISSING] * 3,
-            })
+        })
 
         # Write exact same CSV into cache and export, so no diffs expected
         csv1.to_csv(join(cache_dir, "csv1.csv"), index=False)
@@ -503,10 +490,7 @@ class TestGitArchiveDiffer:
         assert set(new_files) == set()
 
     def test_archive_exports(self, tmp_path):
-        cache_dir = str(tmp_path / "cache")
-        export_dir = str(tmp_path / "export")
-        mkdir(cache_dir)
-        mkdir(export_dir)
+        cache_dir, export_dir = self.set_up(tmp_path)
 
         repo = Repo.init(cache_dir)
         repo.index.commit(message="Initial commit")
@@ -520,7 +504,7 @@ class TestGitArchiveDiffer:
             "missing_val": [Nans.NOT_MISSING] * 3,
             "missing_se": [Nans.NOT_MISSING] * 3,
             "missing_sample_size": [Nans.NOT_MISSING] * 3,
-            })
+        })
 
         # csv1.csv is now a dirty edit in the repo, and to be exported too
         csv1.to_csv(join(cache_dir, "csv1.csv"), index=False)
@@ -563,10 +547,7 @@ class TestGitArchiveDiffer:
         assert repo.active_branch.set_commit("HEAD~1").commit == orig_commit
 
     def test_run(self, tmp_path):
-        cache_dir = str(tmp_path / "cache")
-        export_dir = str(tmp_path / "export")
-        mkdir(cache_dir)
-        mkdir(export_dir)
+        cache_dir, export_dir = self.set_up(tmp_path)
 
         branch_name = "test-branch"
 
@@ -574,13 +555,13 @@ class TestGitArchiveDiffer:
         repo.index.commit(message="Initial commit")
         original_branch = repo.active_branch
 
-        # Set up the current cache to contain `CSVS_BEFORE`.
-        for csv_name, df in CSVS_BEFORE.items():
-            df.to_csv(join(cache_dir, f"{csv_name}.csv"), index=False)
-
-        # Set up the current cache to contain `CSVS_AFTER`.
-        for csv_name, df in CSVS_AFTER.items():
-            df.to_csv(join(export_dir, f"{csv_name}.csv"), index=False)
+        for csv_name, dfs in CSVS.items():
+            # Set up the current cache to contain 'before' files
+            if dfs.before is not None:
+                dfs.before.to_csv(join(cache_dir, f"{csv_name}.csv"), index=False)
+            # Set up the current export to contain 'after' files
+            if dfs.after is not None:
+                dfs.after.to_csv(join(export_dir, f"{csv_name}.csv"), index=False)
 
         # Create and run differ.
         arch_diff = GitArchiveDiffer(
@@ -588,28 +569,15 @@ class TestGitArchiveDiffer:
             branch_name=branch_name, override_dirty=True)
         arch_diff.run()
 
-        # Check that the archive branch contains `CSVS_AFTER`.
+        # Check that the archive branch contains 'after' files.
         arch_diff.get_branch(branch_name).checkout()
-        for csv_name, df in CSVS_AFTER.items():
-            assert_frame_equal(
-                pd.read_csv(join(cache_dir, f"{csv_name}.csv"), dtype=CSV_DTYPES), df)
+        for csv_name, dfs in CSVS.items():
+            if dfs.after is None: continue
+            assert_frame_equal(pd.read_csv(join(cache_dir, f"{csv_name}.csv"), dtype=CSV_DTYPES), dfs.after)
         original_branch.checkout()
 
         # Check exports directory just has incremental changes
-        assert set(listdir(export_dir)) == {"csv1.csv", "csv3.csv", "csv4.csv", "csv5.csv", "csv7.csv", "csv8.csv"}
-        csv1_diff = pd.DataFrame({
-            "geo_id": ["3", "2", "4"],
-            "val": [np.nan, 2.1, 4.0],
-            "se": [np.nan, 0.21, np.nan],
-            "sample_size": [np.nan, 21.0, 40.0],
-            "missing_val": [Nans.DELETED] + [Nans.NOT_MISSING] * 2,
-            "missing_se": [Nans.DELETED] + [Nans.NOT_MISSING] * 2,
-            "missing_sample_size": [Nans.DELETED] + [Nans.NOT_MISSING] * 2,
-            })
-        assert_frame_equal(
-            pd.read_csv(join(export_dir, "csv1.csv"), dtype=CSV_DTYPES),
-            csv1_diff)
-
+        self.check_filtered_exports(export_dir)
 
 class TestFromParams:
     """Tests for creating archive differs from params."""
